# Generated by Django 4.1.5 on 2023-02-03 22:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('scrobbles', '0013_audioscrobblertsvimport_processed_on'),
    ]

    operations = [
        migrations.AddField(
            model_name='audioscrobblertsvimport',
            name='process_log',
            field=models.TextField(blank=True, null=True),
        ),
    ]
