# Generated by Django 4.1.5 on 2023-02-23 15:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0007_sport_default_event_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='sportevent',
            name='thesportsdb_id',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
