# Generated by Django 4.1.5 on 2023-02-03 23:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('scrobbles', '0015_audioscrobblertsvimport_uuid_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='audioscrobblertsvimport',
            name='process_count',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]
