# Generated by Django 4.1.5 on 2023-02-12 22:26

from django.db import migrations, models
import encrypted_field.fields


class Migration(migrations.Migration):

    dependencies = [
        ('profiles', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='lastfm_password',
            field=encrypted_field.fields.EncryptedField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='userprofile',
            name='lastfm_username',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
