# Generated by Django 4.1.5 on 2023-02-25 00:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('scrobbles', '0022_scrobble_book_pages_read'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='audioscrobblertsvimport',
            options={'verbose_name': 'AudioScrobbler TSV Import'},
        ),
        migrations.AlterModelOptions(
            name='koreaderimport',
            options={'verbose_name': 'KOReader Import'},
        ),
        migrations.AlterModelOptions(
            name='lastfmimport',
            options={'verbose_name': 'Last.FM Import'},
        ),
        migrations.AddField(
            model_name='chartrecord',
            name='count',
            field=models.IntegerField(default=0),
        ),
    ]
